/*  Copyright (C) 1996  Byron Stanoszek
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **************************************************************************/

/* snddriv.c */

/* the main program that runs in the background, playing all the music */


#include "sound.h"

int speeds[13]={4410,8820,11025,13230,15435,17640,22050,
                26460,30870,35280,39690,44100,48510};

#define NUMFRAG 24		/* number of fragments in buffer */

int audio;			/* file descriptor of /dev/audio */
int mixer;			/* file descriptor of /dev/mixer */
int sock;			/* file descriptor of /proc/snddriv */
int frag;			/* size of fragments */
unsigned char slvl[NUMFRAG];	/* sound height, for interactive view */
int scselect=1;			/* 1 = scale music down at tune change */
int fragsremaining=0;		/* for output timer purposes */

song_chain *songlist=NULL;	/* whole data songs listed in queue */

int nrows=24;
int ncols=80;

#define GRDL_VER "v1.1a1"


void main(argc, argv)
int argc;
char *argv[];
{
  int a, l, interactive=0, opt=0, chap=0, snum=0;
  char *cmd=*argv;

  /* grab any flags on the command line */
  while(argc > 1) {
    --argc; ++argv;
    if(opt) {
      if(opt & 1)
        chap=atoi(*argv);
      if(opt & 2)
        snum=atoi(*argv);
      opt=0;
      continue;
    }
    if(**argv == '-') {
      for(a=1;(*argv)[a];a++) {
        switch((*argv)[a]) {
        case 'c':
          opt|=1;
          break;
        case 'i':
          interactive=1;
          break;
        case 's':
          opt|=2;
          break;
        case 'v':
          fprintf(stdout, "%s: GRDL Version Reference %s\n", cmd, GRDL_VER);
          fflush(stdout);
          exit(0);
        default:
          fprintf(stderr, "%s: illegal option -- %c\n", cmd, (*argv)[a]);
          fprintf(stderr, "Usage: %s [-iv] [-c chapter] [-s song]\n", cmd);
          fflush(stderr);
          exit(1);
        }
      }
      continue;
    }
  }

  /* open the audio file */
  if((audio = open("/dev/audio", O_WRONLY, 0)) == -1) {
    int err=errno;

    perror("/dev/audio");
    if(err == EBUSY)
      fprintf(stderr, "%s: Another audio program is already using the pcm output device.\n", cmd);
    else
      fprintf(stderr, "%s: Try recompiling the system kernel with the sound driver configured.\n", cmd);
    exit(1);
  }
  a=(NUMFRAG*65536)+10;  /* 1024 bytes per fragment */
  ioctl(audio, SNDCTL_DSP_SETFRAGMENT, &a);
  ioctl(audio, SNDCTL_DSP_GETBLKSIZE, &frag);

  if((mixer = open("/dev/mixer", O_WRONLY, 0)) == -1) {
    perror("/dev/mixer");
    exit(1);
  }

  /* set startup volume */
  input_volume(0x4b4b);

  if(!interactive) {
    /* close standard input */
    fclose(stdin);
    if(fork() != 0)
      exit(0);
    setpgrp();
  } else {
    /* set sound bar levels to 0 */
    for(a=0;a<NUMFRAG;a++)
      slvl[a]=0;
    fix_tty();
  }

  /* ignore system shutdown signals, keep playing :-) */
  signal(SIGTERM, SIG_IGN);

  /* open the input socket */
  if ((sock = open("/proc/snddriv", O_RDONLY|O_NDELAY)) < 0) {
    perror("/proc/snddriv");
    fprintf(stderr, "%s: Try recompiling the system kernel with the snddriv patch installed.\n", cmd);
    exit(1);
  }

  /* load in the startup tune */
  read_song((chap*256)+snum);

#define SONGEND ((songlist->repeat_mode >= 3 && songlist->repeat > 0) ? \
                 songlist->repeat:songlist->length)


  /* main loop */

  while(1) {
    /* if we're repeating the song, start somewhere in the middle */
    if(songlist->c >= SONGEND) {
      songlist->c = (songlist->repeat_mode>=2)?0:songlist->repeat;
      if(songlist->scale != -1) {
        if(songlist->scale < songlist->speed)
          songlist->speed--;
        else
          songlist->speed++;
        if(songlist->scale == songlist->speed)
          songlist->scale = -1;
        ioctl(audio, SNDCTL_DSP_SYNC);
        set_audio(songlist);
        continue;
      }
      if(songlist->repeat_num > -2)
        --songlist->repeat_num;
      if(songlist->fpause) {
        ioctl(audio, SNDCTL_DSP_SYNC);
        if(!(!songlist->repeat_mode || songlist->repeat_mode == 4 ||
           songlist->repeat_num == -1)) {
          read_input();
          continue;
        }
      }
      if(!songlist->repeat_mode || songlist->repeat_mode == 4 ||
         songlist->repeat_num == -1) {
        song_chain *new;

        /* free up song */
        new=songlist->next;
        free(songlist->data);
        free(songlist);

        songlist=new;
        ioctl(audio, SNDCTL_DSP_SYNC);
        if(new)
          set_audio(new);
        else
          read_song(0);
        continue;
      }
    }

    /* play song until end is reached */
    while(1) {
      /* check for sound controls */
      if(!songlist->fpause)
        read_input();

      /* write out more bytes of the song */
      if((l=SONGEND-songlist->c) > frag)
        l = frag;

      songlist->c += write(audio, songlist->data+songlist->c, l);

      if(interactive) {
        if((a=getchar()) != EOF) {
          printf("%d\n", a);
          fflush(stdout);
        }
      }
      /* tune volume scale up or down */
      if(--fragsremaining < 0 && songlist->volume_scale != -1) {
        if(songlist->volume_scale < songlist->volume/1000) {
          songlist->volume-=((songlist->volspeed+1)*10)/(songlist->bytesize);
          if(songlist->volume/1000 <= songlist->volume_scale) {
            songlist->volume=songlist->volume_scale*1000;
            songlist->volume_scale = -1;
          }
        } else {
          songlist->volume+=((songlist->volspeed+1)*10)/(songlist->bytesize);
          if(songlist->volume/1000 >= songlist->volume_scale) {
            songlist->volume=songlist->volume_scale*1000;
            songlist->volume_scale = -1;
          }
        }
        a=(((songlist->volume/1000)+50)*256)+(songlist->volume/1000)+50;
        ioctl(mixer, SOUND_MIXER_WRITE_PCM, &a);
      }

      if(songlist->c >= SONGEND)
        break;
    }
  }
}

int read_song(track)
int track;
{
  song_chain *new;
  wave_header head;
  char soundfile[256];
  int a, fd, typ;

  /* read new song */
  sprintf(soundfile, "%s/ch%02x/%02x.wv2", SOUND_LIBRARY, track/256, track%256);
  if((fd = open(soundfile, O_RDONLY, 0)) == -1) {
    sprintf(soundfile, "%s/ch%02x/%02x.wav", SOUND_LIBRARY,
            track/256, track%256);
    if((fd = open(soundfile, O_RDONLY, 0)) == -1)
      typ=2; /* blank input */
    else
      typ=1; /* .wav input */
  } else
    typ=0; /* .wv2 input */

  /* open new song pointer */
  new=(song_chain *)malloc(sizeof(song_chain));
  new->next = songlist;
  songlist=new;

  /* read header from file */
  switch(typ) {
  case 2:
    new->stereo = 0;
    new->speed = 24;
    new->sample_size = 8;
    new->bytesize = 1;
    new->length = 1024;
    new->repeat = 0;
    new->data = malloc(1024);
    for(a=0;a<1024;a++)
      *(new->data+a)=128;
    new->repeat_mode = 1;
    new->volume = 0;
    break;
  default:
    read(fd, &head, sizeof(head));
    new->stereo = (head.modus == 2)?1:0;
    new->speed = speed_to_scale(head.sample_fq);
    new->sample_size = (int)head.bit_p_spl;
    new->bytesize = (int)head.byte_p_spl;
    new->length = head.data_length;
/*  new->repeat = head.data_repeat;
    if(new->repeat > new->length) */
      new->repeat=0;
    new->data = malloc(new->length);
    read(fd, new->data, new->length);
    close(fd);
    new->repeat_mode = 1;
    new->volume = 50000;
  }

  new->default_speed = new->speed;
  new->default_volume = new->volume;
  new->repeat_num = -1;
  new->fpause = 0;
  new->scale = -1;
  new->volspeed = 255;
  new->volume_scale = -1;
  new->scale_toggle = 1;

  new->tune = track;
  new->c = 0;

  set_audio(new);
  return 1;
}

void set_audio(song)
song_chain *song;
{
  int a;

  /* set startup volume */
  a=(((song->volume/1000)+50)*256)+(song->volume/1000)+50;
  ioctl(mixer, SOUND_MIXER_WRITE_PCM, &a); /* output volume */

  /* clear existing sound audio */
  ioctl(audio, SNDCTL_DSP_RESET);

  ioctl(audio, SNDCTL_DSP_SAMPLESIZE, &song->sample_size);
  ioctl(audio, SNDCTL_DSP_STEREO, &song->stereo);

  /* set new speed */
  a=speeds[song->speed-18];
  ioctl(audio, SNDCTL_DSP_SPEED, &a);

  fragsremaining=NUMFRAG;
}

void read_input()
{
  song_chain *new=NULL, *snext;
  static int pc=0, queue=0;
  static unsigned char port[4];
  int a, l, arg=0;

  if(songlist->fpause)
    songlist->fpause=0;

  if(queue >= 0)
    queue--;
  /* process input data */
  while((l=read(sock, port+pc, 1)) == 1) {
    queue=2;

    switch(*port) {
    case WV_PING:
      set_audio(songlist);
      pc=-1;
      break;
    case WV_SCSELECT_OFF:
      scselect=0;
      pc=-1;
      break;
    case WV_SCSELECT_ON:
      scselect=1;
      pc=-1;
      break;
    case WV_INPUT_OFF:
      input_volume(0);
      pc=-1;
      break;
    case WV_INPUT_ON:
      input_volume(0x4b4b);
      pc=-1;
      break;
    case WV_DEFAULT:
      scselect=1;
      songlist->repeat_mode=1;
      songlist->speed=songlist->default_speed;
      songlist->scale=-1;
      songlist->volume=songlist->default_volume;
      songlist->volume_scale=-1;
      songlist->scale_toggle=1;
      a=(((songlist->volume/1000)+50)*256)+(songlist->volume/1000)+50;
      ioctl(mixer, SOUND_MIXER_WRITE_PCM, &a);
      set_audio(songlist);
      pc=-1;
      break;
    case WV_SPEED:
      if(pc == 1) {
        a=speed_to_scale(*(port+1)*1000);
        songlist->scale=-1;
        if(songlist->speed != a) {
          songlist->speed = a;
          set_audio(songlist);
        }
        pc=-1;
      }
      break;
    case WV_SCALE:
      if(pc == 1) {
        songlist->scale=*(port+1);
        if(songlist->scale < 18)
          songlist->scale = 18;
        if(songlist->scale > 30)
          songlist->scale = 30;
        if(songlist->scale == songlist->speed)
          songlist->scale = -1;
        pc=-1;
      }
      break;
    case WV_NORMAL:
      songlist->scale=-1;
      if(songlist->speed != songlist->default_speed) {
        songlist->speed=songlist->default_speed;
        set_audio(songlist);
      }
      pc=-1;
      break;
    case WV_FREE:
      if(songlist->next) {
        new=songlist->next;
        free(songlist->data);
        free(songlist);
        songlist=new;
        set_audio(new);
      }
      pc=-1;
      break;
    case WV_ALLOC:
    case WV_ALLOCFX:
    case WV_FADEFX:
    case WV_HALFFX:
    case WV_HALFFXSPEED:
    case WV_ALLOCMIX:
      if(pc == ((*port==WV_HALFFXSPEED)?3:(*port==WV_ALLOCMIX)?1:2)) {
        if(*port==WV_FADEFX || *port==WV_HALFFX || *port==WV_HALFFXSPEED) {
          if(songlist->volume_scale == -1)
            songlist->volume_scale=songlist->volume/1000;
          songlist->volume=0;
        }
        if(*port == WV_ALLOCMIX)
          arg=*(port+1)+65280;
        else {
          arg=*(port+2);
          arg+=(*(port+1))*256;
        }
        read_song(arg);
        if(*port != WV_ALLOC) {
          songlist->fpause=1;
          songlist->repeat_mode=(*port==WV_HALFFX || *port==WV_HALFFXSPEED)?4:0;
          if(pc == 3 && *(port+3) > 17 && *(port+3) < 31 &&
             songlist->speed != *(port+3)) {
            songlist->speed=*(port+3);
            set_audio(songlist);
          }
        }
        pc=-1;
      }
      break;
    case WV_RESTART:
    case WV_MIDWAY:
      ioctl(audio, SNDCTL_DSP_RESET);
      if(*port == WV_RESTART)
        songlist->c=0;
      else
        songlist->c=songlist->repeat;
      pc=-1;
      break;
    case WV_SELECT:
    case WV_PREV:
    case WV_NEXT:
    case WV_OFF:
      if(*port != WV_SELECT)
        pc=2;
      if(pc != 2)
        break;

      for(new=songlist;new;new=snext) {
        if(!(snext=new->next))
          break;
        free(new->data);
        free(new);
      }

      if(*port == WV_SELECT) {
        arg=*(port+2);
        arg+=(*(port+1))*256;
      } else {
        arg=new->tune;
        if(*port != WV_SELECT) {
          arg = new->tune;
          if(*port == WV_PREV)
            arg--;
          if(*port == WV_NEXT)
            arg++;
          if(*port == WV_OFF || arg < 0 || arg > 65535)
            arg=0;
        }
      }

      if(arg == new->tune) {
        if(new != songlist) {
          set_audio(new);
          songlist=new;
        }
        pc=-1;
        break;
      }
      free(new->data);
      free(new);
      songlist=NULL;
      read_song(arg);
      pc=-1;
      break;
    case WV_REPEATOFF:
      songlist->repeat_mode=0;
      pc=-1;
      break;
    case WV_REPEATON:
      songlist->repeat_mode=1;
      pc=-1;
      break;
    case WV_REPEATFULL:
      songlist->repeat_mode=2;
      pc=-1;
      break;
    case WV_REPEATMIDDLE:
      songlist->repeat_mode=3;
      pc=-1;
      break;
    case WV_REPEATEXIT:
      songlist->repeat_mode=4;
      pc=-1;
      break;
    case WV_REPEATNUM:
      if(pc == 1) {
        songlist->repeat_num=*(port+1);
        pc=-1;
      }
      break;
    case WV_REPEATINF:
      songlist->repeat_num=-1;
      pc=-1;
      break;
    case WV_VOLUME:
      if(pc == 1) {
        songlist->volume_scale=-1;
        if(*(port+1) > 50)
          songlist->volume=50000;
        else
          songlist->volume=*(port+1)*1000;
        a=(((songlist->volume/1000)+50)*256)+(songlist->volume/1000)+50;
        ioctl(mixer, SOUND_MIXER_WRITE_PCM, &a);
        pc=-1;
      }
      break;
    case WV_VOLOFF:
    case WV_VOLDOWN:
    case WV_VOLSOFT:
    case WV_VOLMAX:
      a=*port == WV_VOLOFF?0:*port == WV_VOLDOWN?10:*port == WV_VOLSOFT?38:50;
      if(songlist->scale_toggle) {
        songlist->volume_scale=a;
      } else {
        songlist->volume_scale=-1;
        songlist->volume=a*1000;
        a=(((songlist->volume/1000)+50)*256)+(songlist->volume/1000)+50;
        ioctl(mixer, SOUND_MIXER_WRITE_PCM, &a);
      }
      pc=-1;
      break;
    case WV_VOLSCALE:
      if(pc == 1) {
        songlist->volume_scale=*(port+1);
        if(songlist->volume_scale > 50)
          songlist->volume_scale = 50;
        pc=-1;
      }
      break;
    case WV_VOLSPEED:
      if(pc == 1) {
        unsigned char xfer=*(port+1);
        songlist->volspeed=xfer;
        pc=-1;
      }
      break;
    case WV_SCALEOFF:
      songlist->scale_toggle=0;
      pc=-1;
      break;
    case WV_SCALEON:
      songlist->scale_toggle=1;
      pc=-1;
      break;
    case WV_RESET:
      ioctl(audio, SNDCTL_DSP_RESET);
      pc=-1;
      break;
    case WV_SYNC:
      ioctl(audio, SNDCTL_DSP_SYNC);
      pc=-1;
      break;
    case WV_WAIT:
      if(pc == 1) {
        sleep(*(port+1));
        pc=-1;
      }
      break;
    case WV_FINISH:
      songlist->fpause=1;
      pc=-1;
      break;
    default:
      if(*port > 17 && *port < 31) {
        songlist->scale=-1;
        if(songlist->speed != *port) {
          songlist->speed=*port;
          set_audio(songlist);
        }
      }
      pc=-1;
    }
    pc++;
    if(songlist->fpause)
      break;
  }

  if(!queue) {
    printf("\e[v"); /* send code to reset /proc/snddriv */
    fflush(stdout);
    lseek(sock, 0L, SEEK_SET);
  }
}

/* change speed in Hz to a number in the scale table speeds[] */
int speed_to_scale(hertz)
int hertz;
{
  int a;

  for(a=0;a<12;a++)
    if(hertz <= speeds[a]+2000)
      return a+18;
  return 30;
}

/* put terminal into cbreak mode */
void fix_tty()
{
  struct termio pty;
  char *ptr;
  int a;

  fflush(stdout);
  ioctl(0, TCGETA, &pty);
  pty.c_lflag &= ~ICANON;
  pty.c_lflag &= ~ECHO;
  pty.c_iflag &= ~IXON;
  pty.c_iflag &= ~IXOFF;
  ioctl(0, TCSETA, &pty);

  /* alter lines and columns if available */
#ifdef TIOCGWINSZ
  {
    struct winsize ws;

    if(ioctl(1, TIOCGWINSZ, &ws) != -1) {
      if(ws.ws_row > 22 && ws.ws_row < 251)
        nrows = ws.ws_row;
        if(ws.ws_col > 79 && ws.ws_col < 501)
          ncols = ws.ws_col;
    }
  }
#endif

  ptr=getenv("LINES");
  if(ptr && (a=atoi(ptr)))
    if(a > 22 && a < 251)
      nrows=a;
  ptr=getenv("COLUMNS");
  if(ptr && (a=atoi(ptr)))
    if(a > 79 && a < 501)
      ncols=a;
}

void input_volume(amt)
int amt;
{
  int a=amt;

  ioctl(mixer, SOUND_MIXER_WRITE_CD, &a);
  ioctl(mixer, SOUND_MIXER_WRITE_MIC, &a);
  ioctl(mixer, SOUND_MIXER_WRITE_LINE, &a);
  ioctl(mixer, SOUND_MIXER_WRITE_LINE1, &a);
  ioctl(mixer, SOUND_MIXER_WRITE_LINE2, &a);
  ioctl(mixer, SOUND_MIXER_WRITE_LINE3, &a);
}
